package com.foreverwin.mesnac.quartz.job;

import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.mesnac.equip.service.ResourceInspectTaskJobService;
import com.foreverwin.mesnac.equip.service.ResourceInspectTaskService;
import com.foreverwin.modular.core.exception.BaseException;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * @Description TODO
 * @Author zhaojiawei
 * @Since 2021-07-22
 */
@DisallowConcurrentExecution
public class ResourceInspectTaskJob implements Job {
    private static final String SITE_UPPERCASE = "SITE";
    private static final String SITE_LOWERCASE = "site";

    private Logger logger = LoggerFactory.getLogger("ResourceInspectTaskJob");

    @Autowired
    private ResourceInspectTaskJobService resourceInspectTaskJobService;

    @Override
    public void execute(JobExecutionContext context) throws JobExecutionException {
        logger.info("设备点检任务创建定时任务开始执行");

        //获取站点，若站点为空则抛出异常
        String site = getSite(context);
        if (StringUtil.isBlank(site)) {
            throw new BaseException("equip.quartz.task.not.site", null);
        }

        resourceInspectTaskJobService.resourceInspectTaskJobProcess(site);
    }

    private String getSite(JobExecutionContext context) {
        String site = context.getJobDetail().getJobDataMap().getString(SITE_LOWERCASE);
        if (StringUtil.isBlank(site)) {
            site = context.getJobDetail().getJobDataMap().getString(SITE_UPPERCASE);
        }
        return site;
    }
}
